/*
===============================================================================
 Name        : main.c
 Author      : $(author)
 Version     :
 Copyright   : $(copyright)
 Description : main definition
===============================================================================
*/

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif

#include <cr_section_macros.h>

#include "timer32.h"
#include "timer16.h"
#include "string.h"
#include "type.h"
#include "gpio.h"
#include "uart.h"
#include "ssp.h"
#include "i2c.h"
#include "stdlib.h"
#include "mp3_11.h"
#include "vs1063.h"

// FatFS
#include "integer.h"
#include "diskio.h"
#include "ffconf.h"
#include "ff.h"

#define DELAY_TIMER_NUM		1		/* 0 or 1 for 32-bit timers only */
#define SSP_SD				0

#define ENCDEC_BUF_SIZE			(512*6)
//#define ENCDEC_BUF_SIZE			(512*1)

#define NUM_MAINMENU			4
#define	MAINMENU_RECORD			0
#define	MAINMENU_REPLAY			1
#define	MAINMENU_SETRTC			2
#define MAINMENU_CLRALRM		3
#define MAINMENU_ENCSETTING		4

// ボタン操作
#define SW_PRESSED				0x80
#define SWPREV_PRESSED     		0x80
#define SWNEXT_PRESSED          0x81
#define SWSTART_PRESSED         0x82
#define SWUP_PRESSED            0x83
#define SWDOWN_PRESSED          0x84
#define SWMODE_PRESSED     		0x85
#define SWPOFF_PRESSED			0x88
#define SWSELECT_PRESSED      	SWSTART_PRESSED

// 機能
#define PLAY_NEXT_MP3           0x91
#define PLAY_PREV_MP3           0x92
#define PLAY_OTHER_DIR          0x93
#define POWEROFF                0x94
#define GOTO_MAINMENU           0x95
#define SELECT_OTHER_MP3		0x96
#define CHANGE_ITEMS			0x97
#define DO_NOTHING				0x98
#define STOP_RECORD				0x99

// POWER ON の維持
#define PORT_HOLD_POWER			1
#define BIT_HOLD_POWER			24

// SW/POWERの押下検出
#define PORT_SENSE_POWER_DOWN	0
#define BIT_SENSE_POWER_DOWN	7

// リレー駆動
#define PORT_RELAY_ON		1
#define BIT_RELAY_ON		7

// 外部出力
#define PORT_OUTSIDE		1
#define BIT_OUTSIDE			19

// SW-0
#define PORT_SW_IN_0		1
#define BIT_SW_IN_0			27
// SW-1
#define PORT_SW_IN_1		1
#define BIT_SW_IN_1			26
// SW-2
#define PORT_SW_IN_2		0
#define BIT_SW_IN_2			11
// SW-3
#define PORT_SW_IN_3		0
#define BIT_SW_IN_3			12
// SW-4
#define PORT_SW_IN_4		0
#define BIT_SW_IN_4			13

#define PORT_ON_BOARD_LED		1
#define BIT_ON_BOARD_LED		6

// 履歴ファイル HISTFILE parameters
#define HISTFILE                "/HIST04.DAT"
#define RECLEN_HISTFILE         10UL
#define HS_LASTDIR              0
#define HS_LASTTRACK            2
#define HS_LASTBYTE             4
#define HS_NEXTRANDOMDIR        8

//#define ENCODING_GAIN   (1024*9)        // original=8
#define ENCODING_GAIN   (200)

#define	ENCPATHESFILE       "/ENCPAT05.DAT"     // VS1063A PATCHES / Encoding only, compressed plugin
#define	PATHESFILE          "/PATCH05.DAT"      // VS1063A PATCHES / Full Patch
#define SCHEDULEFILE        "/SCHDL05.TXT"      // record schedule

// MP3ファイルリスト SNDFILELISTFILE parameters
#define SNDFILELISTFILE     "SNDLST11.DAT"
#define RECLEN_MP3LIST		16UL

// EEPROM Read/Write
#define EEPROM_ADDR_SCHEDULE    64
#define EEPROM_LEN_SCHEDULE    	64
#define EEPROM_ADDR_LASTMP3FILENAME		128
//#define EEPROM_ADDR_ENCSETTING	160
#define IAP_ADDRESS				0x1FFF1FF1
#define EEPROM_WRITE			61
#define EEPROM_READ				62
typedef unsigned int (*IAP)(unsigned int[], unsigned int[]);
static const IAP iap_entry = (IAP) IAP_ADDRESS;

static FATFS        Fatfs;
static uint8_t      Mp3Buff[ENCDEC_BUF_SIZE];			// MP3データバッファ
static uint8_t      PlayDirHist[RECLEN_HISTFILE];		// 履歴ファイルレコード
static uint8_t		RelayStatus;
static uint8_t		LedStatus;
volatile uint8_t	PinEvent;
static uint16_t     CHATTERING_TMR;
uint16_t			Volume;			// 音量
volatile uint16_t	TimerCounter16_0;
volatile uint16_t	TimerCounter16_ms;
volatile uint16_t	TimerCounter16_s;		// 秒カウンタ
static uint16_t		RecordLimit_s;			// 録音時間制限
//static uint8_t		CurEncSetting;

//static char		*EncFormatStr[] = {"MP3/LEFT        ", "MP3/MONO        "};
//static uint16_t	EncFormatTab[] = {0x62, 0x64};		// mp3/left channel, mp3/mono
//static uint16_t ClockfTab[] = {0x0000, 0x2000, 0x6000, 0x8000, 0xa000, 0xe000};
//static uint16_t	SamplerateTab[] = {12000, 16000, 24000, 32000, 44100, 48000};
//static uint16_t BitrateTab[] = {24, 32, 40, 48, 64, 80, 96, 128, 160};
#define	NUM_ENCSETTING		12

/*
static uint16_t EncSetting[NUM_ENCSETTING][3] = {
		{0x62, 12000, 24},		// MP3/LEFT, Samplerate, bitrate
		{0x62, 16000, 32},
		{0x62, 16000, 40},
		{0x62, 24000, 48},
		{0x62, 32000, 56},
		{0x62, 32000, 64},
		{0x64, 12000, 24},		// MP3/MONO, Samplerate, bitrate
		{0x64, 16000, 32},
		{0x64, 16000, 40},
		{0x64, 24000, 48},
		{0x64, 32000, 56},
		{0x64, 32000, 64}
};
static char *EncSettingStr[NUM_ENCSETTING] = {
		"LEFT,S12000,B24K  ",
		"LEFT,S16000,B32K  ",
		"LEFT,S16000,B40K  ",
		"LEFT,S24000,B48K  ",
		"LEFT,S32000,B56K  ",
		"LEFT,S32000,B64K  ",
		"MONO,S12000,B24K  ",
		"MONO,S16000,B32K  ",
		"MONO,S16000,B40K  ",
		"MONO,S24000,B48K  ",
		"MONO,S32000,B56K  ",
		"MONO,S32000,B64K  "
};
*/

void InitApp();
void DispMainMenu(uint8_t current_item);
int PauseAndSelectItem();
uint8_t InstantRecording(FIL *file);
uint8_t ScheduledRecording(FIL *file);
uint8_t Encode(char *Mp3FileName, FIL *file, uint8_t CurEncSet, uint16_t recno, uint16_t totalsec);
void SetTimestamp(char *Mp3FileName, uint8_t year, uint8_t month, uint8_t day, uint8_t hour, uint8_t minutes, uint8_t sec);
void AddMp3fileToSndLst(char *Mp3FileName, FIL *file);
uint8_t Replay(FIL *file);
int PlayOneMp3(const char *mp3file, uint16_t dirno, uint8_t trackno);
void LoadPluginFromFile(char *PluginFile, FIL *file);
void InitRtc();
int8_t AdjustRtc();
uint8_t RTC_GetCurrentDateTimeForAdjust(char *buff);
uint8_t RTC_SetDateTime(char *datetime);
uint8_t SetNextSchedule(FIL *file);
void GetEncodeSettingFromEEPROM();
void SaveEncodeSettingToEEPROM();
uint8_t EncodeSetting();
void WaitMs(uint32_t delayInMs);
int iap_eeprom_write(void* eeprom_address, void* ram_address, unsigned int count);
int iap_eeprom_read(void* eeprom_address, void* ram_address, unsigned int count);

int main(void) {
    FRESULT     res;
    FIL 		filMP3;
	int			ret;
    int         return_to_mainmenu;
    uint8_t		selectedmenuitem = 0;

	SystemCoreClockUpdate();

	InitApp();
	//InitRtc();

    PinEvent = 0;
    CHATTERING_TMR = 0;
    LedStatus = 0;
    RecordLimit_s = 2*60*60;		// 2H
	//VS1063_SineTest();

	res = f_mount(&Fatfs, _T("0:"), 0);
	if (res == FR_OK){
		//LoadPluginFromFile(ENCPATHESFILE, &filMP3);

		if (PCF8523_isalarmed() == 1){
			lcd_lprintln(0, "Alarmed         ");
			PCF8523_clearalarm();
			PinEvent = 0;
			// FM10起動、録音終了時刻設定、出力先ファイル名決定、エンコード実行
			GPIOSetBitValue(PORT_OUTSIDE, BIT_OUTSIDE, 1);
			GPIOSetBitValue(PORT_RELAY_ON, BIT_RELAY_ON, 1);
			ScheduledRecording(&filMP3);
			GPIOSetBitValue(PORT_OUTSIDE, BIT_OUTSIDE, 0);
			GPIOSetBitValue(PORT_RELAY_ON, BIT_RELAY_ON, 0);
	    	SetNextSchedule(&filMP3);
	    } else {
	    	SetNextSchedule(&filMP3);

	    	// メインメニュー
    		return_to_mainmenu = TRUE;
        	do {
    	    	DispMainMenu(selectedmenuitem);

				// メインメニュー入力待ち
				ret = PauseAndSelectItem();
				lcd_lprintln(1, "                ");
				switch (ret){
				case SWDOWN_PRESSED:
				case SWNEXT_PRESSED:
					if (selectedmenuitem < (NUM_MAINMENU - 1)){
						selectedmenuitem++;
					} else {
						selectedmenuitem = 0;
					}
					break;
				case SWUP_PRESSED:
				case SWPREV_PRESSED:
					if (selectedmenuitem > 0){
						selectedmenuitem--;
					} else {
						selectedmenuitem = NUM_MAINMENU - 1;
					}
					break;
				case SWSELECT_PRESSED:
					switch (selectedmenuitem){
					case MAINMENU_RECORD:		// 録音
						GPIOSetBitValue(PORT_RELAY_ON, BIT_RELAY_ON, 1);	//
						InstantRecording(&filMP3);
						GPIOSetBitValue(PORT_RELAY_ON, BIT_RELAY_ON, 0);
	       				return_to_mainmenu = FALSE;
						break;
					case MAINMENU_REPLAY:		// 再生
						Replay(&filMP3);
						break;
					case MAINMENU_SETRTC:		// RTC設定
						AdjustRtc();
						break;
					case MAINMENU_CLRALRM:		// アラーム解除
						PCF8523_clearalarm();
						lcd_lprintln(1, "Schedule Cleared");
						break;
					//case MAINMENU_ENCSETTING:	// エンコード設定
					//	EncodeSetting();
					//	break;
					}
					break;
       			case SWPOFF_PRESSED:		// POWER OFF
       				GPIOSetBitValue(PORT_ON_BOARD_LED, BIT_ON_BOARD_LED, 1);
       				return_to_mainmenu = FALSE;
       				break;
				}
        	} while (return_to_mainmenu);
	    }
		RelayStatus = 1;
		GPIOSetBitValue(PORT_RELAY_ON, BIT_RELAY_ON, RelayStatus);
	}
	GPIOSetBitValue(PORT_HOLD_POWER, BIT_HOLD_POWER, 0);	// POWER-DOWN
	while (1){
	}

    return 0 ;
}

void InitApp(){
	//int		i;

	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<10);		// 32bit timer1
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<6);		// Enable AHB clock to the GPIO domain
	LPC_SYSCON->SYSAHBCLKCTRL |= ((1<<19) | (1<<23) | (1<<24));		// Enable AHB clock to the FlexInt, GroupedInt domain

	// PIO1_6 (on board led)
	LPC_IOCON->PIO1_6 = 0x00;			// PIO1_6
	LPC_GPIO->DIR[PORT_ON_BOARD_LED] |= (1 << BIT_ON_BOARD_LED);		// output
	//GPIOSetBitValue(PORT_ON_BOARD_LED, BIT_ON_BOARD_LED, 0);

	// PIO1_24 (HOLD_POWER)
	LPC_IOCON->PIO1_24 = 0;				// PIO1_24
	LPC_GPIO->DIR[PORT_HOLD_POWER] |= (1 << BIT_HOLD_POWER);		// output
	GPIOSetBitValue(PORT_HOLD_POWER, BIT_HOLD_POWER, 1);

	// PIO0_7 (POWER DOWN検出)
	LPC_IOCON->PIO0_7 = 0x10;			// PIO0_7, pull-up enabled
	LPC_GPIO->DIR[PORT_SENSE_POWER_DOWN] &= ~(1 << BIT_SENSE_POWER_DOWN);		// input

	// PIO1_7 (リレー駆動)
	LPC_IOCON->PIO1_7 = 0x0;				// PIO1_7
	//LPC_IOCON->PIO1_7 = 0x400;				// PIO1_7, open-drain mode
	LPC_GPIO->DIR[PORT_RELAY_ON] |= (1 << BIT_RELAY_ON);		// output
	RelayStatus = 0;
	GPIOSetBitValue(PORT_RELAY_ON, BIT_RELAY_ON, RelayStatus);

	// PIO1_19 (外部出力)
	LPC_IOCON->PIO1_19 = 0x00;
	LPC_GPIO->DIR[PORT_OUTSIDE] |= (1 << BIT_OUTSIDE);		// output
	GPIOSetBitValue(PORT_OUTSIDE, BIT_OUTSIDE, 0);

	// PIO1_27 (SW-0)
	LPC_IOCON->PIO1_27 = 0x10;			// PIO1_27, pull-up enabled
	LPC_GPIO->DIR[PORT_SW_IN_0] &= ~(1 << BIT_SW_IN_0);		// input
	LPC_SYSCON->PINTSEL[0] = (BIT_SW_IN_0 + 24);		// channel 0
	NVIC_EnableIRQ(FLEX_INT0_IRQn);
	LPC_GPIO_PIN_INT->ISEL &= ~(0x1 << 0);	/* Edge trigger */
	LPC_GPIO_PIN_INT->IENF |= (0x1 << 0);	/* falling edge */
	LPC_GPIO_PIN_INT->SIENF |= (0x1 << 0);	/* falling edge */

	// PIO1_26 (SW-1)
	LPC_IOCON->PIO1_26 = 0x10;			// PIO1_26, pull-up enabled
	LPC_GPIO->DIR[PORT_SW_IN_1] &= ~(1 << BIT_SW_IN_1);		// input
	LPC_SYSCON->PINTSEL[1] = (BIT_SW_IN_1 + 24);		// channel 1
	NVIC_EnableIRQ(FLEX_INT1_IRQn);
	LPC_GPIO_PIN_INT->ISEL &= ~(0x1 << 1);	/* Edge trigger */
	LPC_GPIO_PIN_INT->IENF |= (0x1 << 1);	/* falling edge */
	LPC_GPIO_PIN_INT->SIENF |= (0x1 << 1);	/* falling edge */

	// PIO0_11 (SW-2)
	LPC_IOCON->TDI_PIO0_11 = 0x91;			// PIO0_11, pull-up enabled, digital mode
	LPC_GPIO->DIR[PORT_SW_IN_2] &= ~(1 << BIT_SW_IN_2);		// input
	LPC_SYSCON->PINTSEL[2] = (BIT_SW_IN_2);		// channel 2
	NVIC_EnableIRQ(FLEX_INT2_IRQn);
	LPC_GPIO_PIN_INT->ISEL &= ~(0x1 << 2);	/* Edge trigger */
	LPC_GPIO_PIN_INT->IENF |= (0x1 << 2);	/* falling edge */
	LPC_GPIO_PIN_INT->SIENF |= (0x1 << 2);	/* falling edge */

	// PIO0_12 (SW-3)
	LPC_IOCON->TMS_PIO0_12 = 0x91;			// PIO0_12, pull-up enabled, digital mode
	LPC_GPIO->DIR[PORT_SW_IN_3] &= ~(1 << BIT_SW_IN_3);		// input
	LPC_SYSCON->PINTSEL[3] = (BIT_SW_IN_3);		// channel 3
	NVIC_EnableIRQ(FLEX_INT3_IRQn);
	LPC_GPIO_PIN_INT->ISEL &= ~(0x1 << 3);	/* Edge trigger */
	LPC_GPIO_PIN_INT->IENF |= (0x1 << 3);	/* falling edge */
	LPC_GPIO_PIN_INT->SIENF |= (0x1 << 3);	/* falling edge */

	// PIO0_13 (SW-4)
	LPC_IOCON->TDO_PIO0_13 = 0x91;			// PIO0_13, pull-up enabled, digital mode
	LPC_GPIO->DIR[PORT_SW_IN_4] &= ~(1 << BIT_SW_IN_4);		// input
	LPC_SYSCON->PINTSEL[4] = (BIT_SW_IN_4);		// channel 4
	NVIC_EnableIRQ(FLEX_INT4_IRQn);
	LPC_GPIO_PIN_INT->ISEL &= ~(0x1 << 4);	/* Edge trigger */
	LPC_GPIO_PIN_INT->IENF |= (0x1 << 4);	/* falling edge */
	LPC_GPIO_PIN_INT->SIENF |= (0x1 << 4);	/* falling edge */

	SSP_IOConfig( SSP_VS );
	SSP_Init( SSP_VS );
	SSP_IOConfig( SSP_SD );		// CLK=PIO0_6, OUT=PIO0_9, IN=PIO0_8, SSEL=PIO0_20
	SSP_Init( SSP_SD );			// CPOL = 0, CPHA = 0, Master, interrupt

	// GPIO
	// VS1063/XCS
	LPC_IOCON->PIO0_18 = 0x00;			// PIO0_18
	LPC_GPIO->DIR[0] |= (1 << 18);		// output
	// VS1063/BSYNC(DCS)
	LPC_IOCON->PIO0_19 = 0x00;			// PIO0_19
	LPC_GPIO->DIR[0] |= (1 << 19);		// output
	// VS1063/DREQ
	LPC_IOCON->PIO0_17 = 0x10;			// PIO0_17, pull-up
	LPC_GPIO->DIR[0] &= ~(1 << 17);		// input
	// VS1011/XRESET
	LPC_IOCON->PIO1_18 = 0x00;			// PIO1_18
	LPC_GPIO->DIR[1] |= (1 << 18);		// output

	//init_timer16(0, 0);
	//LPC_SYSCON->SYSAHBCLKCTRL |= (1<<7);		// 16bit timer0 (CT16B0)
    //NVIC_EnableIRQ(TIMER_16_0_IRQn);
	//LPC_CT16B0->EMR = 0;				// external match register
    // 16bit Timer0
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<7);		// 16bit timer0 (CT16B0)
	LPC_CT16B0->EMR = 0;				// external match register
    //LPC_CT16B0->MCR = 0x07;		/* stop timer on match */
    //LPC_CT16B0->MCR = 0x04;		/* stop timer on match */
	LPC_CT16B0->MCR = (0x3<<0);
    LPC_CT16B0->MR0 = 48000;			// match register = 48000
    //LPC_CT16B0->MR0 = 0xFFFF;			// match register = 65535
    LPC_CT16B0->PR = 0x00;				// prescale register = 0
    //LPC_CT16B0->PR = 0x20;				// prescale register = 32
    LPC_CT16B0->CCR = 0;				// capture control register
    NVIC_EnableIRQ(TIMER_16_0_IRQn);
    //NVIC_DisableIRQ(TIMER_16_0_IRQn);
    LPC_CT16B0->TCR = 0x01;				//

	if ( I2CInit( 0x01 ) == FALSE ){	/* initialize I2c */
	}
    lcd_init(0x27);
	//lcd_puts("initApp ");

    //GetEncodeSettingFromEEPROM();
	VS1063_BSYNC_H();		// BSYNC(DCS)
	VS1063_XCS_H();			// XCS
    VS1063_HardwareReset();

    // GPIO PIN 割り込みがここまでに発生している(MP3_09の例では)。それをここでリセット
    PinEvent = 0;
}

void DispMainMenu(uint8_t current_item){
	lcd_send_cmd(0x80);	// move to 1st line
	switch (current_item){
	case MAINMENU_RECORD:
		lcd_puts((unsigned char *)"1.Start Record    ");
		break;
	case MAINMENU_REPLAY:
		lcd_puts((unsigned char *)"2.Replay Record   ");
		break;
	case MAINMENU_SETRTC:
		lcd_puts((unsigned char *)"3.Set RTC         ");
		break;
	case MAINMENU_CLRALRM:
		lcd_puts((unsigned char *)"4.Reset Alarm     ");
		break;
	case MAINMENU_ENCSETTING:
		lcd_puts((unsigned char *)"5.Encode Setting  ");
		break;
	}
}

// 選択画面における入力待ち
int PauseAndSelectItem(){
	unsigned int		ret;

	ret = 0;
	PinEvent = 0;
    TimerCounter16_s = 0;
	do {
		//__WFI();		// wait for interrupt
		if (PinEvent != 0){
			ret = (unsigned int)PinEvent;
			PinEvent = 0;
			return ret;
		}

		if (TimerCounter16_s == 30){
			return SWPOFF_PRESSED;
		}
	} while (1);
}

uint8_t Decode(const char *Mp3FileName, FIL *file, uint16_t bytespersec){
    UINT        bytes, sendbytes, sentbytes;
    //uint8_t     i;
    FRESULT     res;
    uint8_t     ret;
    uint16_t	mode;
    int         continue_flag;

    while(!VS1063_DREQ()) ;
    mode = SM_SDINEW_B;
    VS1063_WriteSci(SCI_MODE, mode);

    while(!VS1063_DREQ()) ;
    //mode = 0x4000;      // x2.5
    mode = (0x2000 | CLOCKF_FREQ);      // enough to decode MP3 (x2.0)
    //mode = (0x8000 | CLOCKF_FREQ);     // = 0x8000 + ((12000000 - 8000000)/4000)	// VS1063
    //mode = 0x1000;      // too slow
    VS1063_WriteSci(SCI_CLOCKF, mode);          // CLOCKF

    //while(!VS1063_DREQ()) ;
    //VS1063_WriteSci(SCI_AIADDR, 0x50);         // Activation(Patch利用時)

    ret = 0;
    continue_flag = 1;
    res = f_open(file, Mp3FileName, FA_READ | FA_OPEN_EXISTING);
    while (continue_flag){
        res = f_read(file, Mp3Buff, ENCDEC_BUF_SIZE, &bytes);
        if (res != FR_OK || bytes == 0) break;
        sentbytes = 0;
        while (bytes > 0){
            sendbytes = (bytes > 32 ? 32 : bytes);
            while(!VS1063_DREQ()) ;
            VS1063_DCS_L();
        	SSP_Send(SSP_VS, Mp3Buff + sentbytes, sendbytes);
            VS1063_DCS_H();
            bytes -= sendbytes;
            sentbytes += sendbytes;
        }

    	switch (PinEvent){
    	case SWPOFF_PRESSED:
    		continue_flag = 0;
    		break;
    	case SWPREV_PRESSED:
    	case SWNEXT_PRESSED:
    	case SWSTART_PRESSED:
    	case SWMODE_PRESSED:
			PinEvent = 0;
    		break;
    	default:
	    	break;
    	}
    }
    f_close(file);

    return ret;
}

/*
void GetEncodeSettingFromEEPROM(){
	uint8_t		buff[8];

	iap_eeprom_read(EEPROM_ADDR_ENCSETTING, buff, 1);

	CurEncSetting = buff[0];

	// 保存されている設定がない
	if (!(CurEncSetting >= 0 && CurEncSetting < NUM_ENCSETTING)){
		CurEncSetting = 0;
	}
}

void SaveEncodeSettingToEEPROM(){
	uint8_t		buff[8];

	buff[0] = CurEncSetting;
	iap_eeprom_write(EEPROM_ADDR_ENCSETTING, buff, 1);
}
*/

// スケジュール化された録音ではない、即席録音
uint8_t InstantRecording(FIL *file){
	uint8_t	year, month, day, hour, minutes, sec;
	uint8_t	fileno;
	uint8_t	ret;
	char	mp3file[16], lastmp3file[16];

    PCF8523_gettime(&year, &month, &day, &hour, &minutes, &sec);

	// 前回録音のMP3ファイル名をEEPROMから取得
	iap_eeprom_read(EEPROM_ADDR_LASTMP3FILENAME, lastmp3file, 13);

	// 出力ファイル名
	mp3file[0] = ((year / 10) % 10) + '0';
	mp3file[1] = (year % 10) + '0';
	mp3file[2] = ((month / 10) % 10) + '0';
	mp3file[3] = (month % 10) + '0';
	mp3file[4] = ((day / 10) % 10) + '0';
	mp3file[5] = (day % 10) + '0';
	if (memcmp(mp3file, lastmp3file, 6) == 0){		// 年月日の部分が同じなら連番を+1
		fileno = atoi(&lastmp3file[6]) + 1;
	} else {
		fileno = 1;
	}
	mp3file[6] = ((fileno / 10) % 10) + '0';
	mp3file[7] = (fileno % 10) + '0';
	mp3file[8] = '.';
	mp3file[9] = 'M';
	mp3file[10] = 'P';
	mp3file[11] = '3';
	mp3file[12] = '\0';

	lcd_lprintln(1, mp3file);
	//Delay(3000);

	// 今回のMP3ファイル名をEEPROMに保存
	//memcpy(lastmp3file, mp3file, 13);
	iap_eeprom_write(EEPROM_ADDR_LASTMP3FILENAME, mp3file, 13);

	ret = Encode(mp3file, file, 0, 0, 0);
	SetTimestamp(mp3file, year, month, day, hour, minutes, sec);

	return ret;
}

// 予約録音実行
uint8_t ScheduledRecording(FIL *file){
	uint8_t	buff[EEPROM_LEN_SCHEDULE];
	uint8_t	year_sch, month_sch, day_sch, hour_sch, minutes_sch;
	uint8_t	year, month, day, hour, minutes, sec;
	uint8_t	fileno;
    uint8_t ret;
	char	mp3file[16], lastmp3file[16];

	lcd_lprintln(1, "Scheduled Rec   ");
    PCF8523_gettime(&year, &month, &day, &hour, &minutes, &sec);

    // EEPROMに保存されている録音スケジュールを取得
    memset(buff, 0, EEPROM_LEN_SCHEDULE);
	iap_eeprom_read(EEPROM_ADDR_SCHEDULE, buff, EEPROM_LEN_SCHEDULE);
	// 録音終了日時
	year_sch = atoi((char *)&buff[15]);
	month_sch = atoi((char *)&buff[18]);
	day_sch = atoi((char *)&buff[21]);
	hour_sch = atoi((char *)&buff[24]);
	minutes_sch = atoi((char *)&buff[27]);

	// 終了時刻をアラームにセット
	if ((year_sch == year && month_sch == month && day_sch == day && hour_sch == hour && minutes_sch >= minutes) ||
		(year_sch == year && month_sch == month && day_sch == day && hour_sch > hour) ||
		(year_sch == year && month_sch == month && day_sch > day) ||
		(year_sch == year && month_sch > month) ||
		(year_sch > year) ){	// 現在時刻より後の

		PCF8523_setalarm(day_sch, hour_sch, minutes_sch);
	} else {
		lcd_lprintln(1, "Invalid end time");
		WaitMs(3000);
		return DO_NOTHING;
	}

	// 前回のMP3ファイル名
	iap_eeprom_read(EEPROM_ADDR_LASTMP3FILENAME, lastmp3file, 13);

	// 出力ファイル名(YYMMDD+連番.MP3)
	mp3file[0] = ((year / 10) % 10) + '0';
	mp3file[1] = (year % 10) + '0';
	mp3file[2] = ((month / 10) % 10) + '0';
	mp3file[3] = (month % 10) + '0';
	mp3file[4] = ((day / 10) % 10) + '0';
	mp3file[5] = (day % 10) + '0';
	if (memcmp(mp3file, lastmp3file, 6) == 0){		// 同じ年月日
		fileno = atoi(&lastmp3file[6]) + 1;			// 連番を＋１ (lastmp3fileは拡張子を含むがatoiは期待通りの動作をする)
	} else {
		fileno = 1;
	}
	mp3file[6] = ((fileno / 10) % 10) + '0';
	mp3file[7] = (fileno % 10) + '0';
	mp3file[8] = '.';
	mp3file[9] = 'M';
	mp3file[10] = 'P';
	mp3file[11] = '3';
	mp3file[12] = '\0';

	lcd_lprintln(1, mp3file);
	//Delay(3000);

	// 今回のMP3ファイル名をEEPROMに保存
	iap_eeprom_write(EEPROM_ADDR_LASTMP3FILENAME, mp3file, 13);

	ret = Encode(mp3file, file, 0, 0, 0);
	SetTimestamp(mp3file, year, month, day, hour, minutes, sec);

	return ret;
}

// エンコード
uint8_t Encode(char *Mp3FileName, FIL *file, uint8_t CurEncSet, uint16_t recno, uint16_t totalsec){
    uint16_t    mode;
    uint16_t    samplerate;
    uint16_t    aictrl3;
    uint16_t    wramaddr;
    uint16_t    val;
    FRESULT     res;
    int         continue_flag;
    UINT        fillwords, bw;
    //uint8_t     swchk;
    DWORD       filesize;
    uint16_t	readbytes;
    uint16_t    i;
    uint8_t     ret;
    uint8_t		zero_flg;

    ret = FR_OK;
    lcd_lprintln(1, "Encode Start    ");
    //lcd_locate(0, 0);
	//lcd_puts((unsigned char *)"Encode Start");

	LoadPluginFromFile(ENCPATHESFILE, file);

    while(!VS1063_DREQ()) ;
    //mode = (0x0000 | CLOCKF_FREQ);      // left channel,bitrate=32k,samplerate=16kのエンコードには不足、bitrate=24,samplerate=12kは可能
    //mode = (0x2000 | CLOCKF_FREQ);      // left channel,bitrate=32k,samplerate=16kのエンコード可能
    //mode = (0x6000 | CLOCKF_FREQ);      // enough for mono encoding
    //mode = (0x8000 | CLOCKF_FREQ);        // experiment
    mode = (0xa000 | CLOCKF_FREQ);        // experiment
    //mode = (0xe000  | CLOCKF_FREQ);;      // full spec.
    //mode = ClockfTab[CurEncSet];
    VS1063_WriteSci(SCI_CLOCKF, mode);          // CLOCKF

    while(!VS1063_DREQ()) ;
    //samplerate = 48000UL;
    //samplerate = 44100UL;
    //samplerate = 32000UL;
    //samplerate = 24000UL;
    //samplerate = 16000UL;
    samplerate = 12000UL;
    //samplerate = SamplerateTab[CurEncSet];
    //samplerate = EncSetting[CurEncSet][1];
    VS1063_WriteSci(SCI_AICTRL0, samplerate);

    while(!VS1063_DREQ()) ;
    //VS1063_WriteSci(SCI_AICTRL1, 0);           // encoding gain
    VS1063_WriteSci(SCI_AICTRL1, ENCODING_GAIN);           // encoding gain

    while(!VS1063_DREQ()) ;
    //aictrl3 = 0x60;         // mp3/stereo
    //aictrl3 = 0x64;         // mp3/mono
    //aictrl3 = 0x63;         // mp3/right channel
    aictrl3 = 0x62;         // mp3/left channel
    //aictrl3 = EncFormatTab[CurEncSet];
    //aictrl3 = EncSetting[CurEncSet][0];
    VS1063_WriteSci(SCI_AICTRL3, aictrl3);

    while(!VS1063_DREQ()) ;
	wramaddr = 0;
	//wramaddr |= RQ_MODE_CBR;
	//wramaddr |= RQ_MODE_VBR;
	//↓Bitrate
	wramaddr |= RQ_MULT_1000;
	wramaddr |= 24;
	//wramaddr |= 32;
	//wramaddr |= 40;
	//wramaddr |= 48;
	//wramaddr |= 64;
	//wramaddr |= 80;
	//wramaddr |= 96;
	//wramaddr |= 128;
	//wramaddr |= 160;
	//wramaddr |= BitrateTab[CurEncSet];
	//wramaddr |= EncSetting[CurEncSet][2];
	VS1063_WriteSci(SCI_WRAMADDR, wramaddr);

    while(!VS1063_DREQ()) ;
    val = VS1063_ReadSci(SCI_MODE);
    //val |= (SM_ENCODE_B | SM_LINE1_B);
    val = (SM_ENCODE_B | SM_LINE1_B | SM_SDINEW_B);
    VS1063_WriteSci(SCI_MODE, val);

    while(!VS1063_DREQ()) ;
    VS1063_WriteSci(SCI_AIADDR, 0x50);         // Activation

    res = f_open(file, Mp3FileName, FA_WRITE | FA_CREATE_ALWAYS);

    if (res == FR_OK){
        lcd_lprintln(0, "Recording Start ");
        //WriteLog("recording write start", 0);

        continue_flag = 1;
		PinEvent = 0;
        filesize = 0;
        readbytes = 0;
        TimerCounter16_ms = 0;
        TimerCounter16_s = 0;
        zero_flg = 0;
        fillwords = 0;

    	//lcd_send_cmd(0xC0);	// move to 2nd line
        while (continue_flag){
            while(!VS1063_DREQ()) ;
            fillwords = VS1063_ReadSci(SCI_HDAT1);
            if (fillwords > 0) {
				for (i=0; i<fillwords; i++){
					//while(!VS1063_DREQ()) ;        // 不要
					val = VS1063_ReadSci(SCI_HDAT0);
					Mp3Buff[readbytes++] = (val >> 8);
					Mp3Buff[readbytes++] = (val & 0x00FF);
					if (readbytes == ENCDEC_BUF_SIZE){
						res = f_write(file, Mp3Buff, ENCDEC_BUF_SIZE, &bw);
						readbytes = 0;
						filesize += ENCDEC_BUF_SIZE;
					}
				}
				// ↓VS1063の内部ではバッファのクリア処理が動いていると思われるが、待ちを入れることでクリア処理が終わる前に読み出しされてしまうことを避ける。
				//WaitMs(100);
				//WaitMs(50);
				WaitMs(500);
				zero_flg = 1;
            } else {
				WaitMs(5);
            //} else if (zero_flg == 1) {
            	zero_flg = 0;
            	/*
				if (!(VS1063_ReadSci(SCI_MODE) & SM_CANCEL_B) &&
						!VS1063_ReadSci(SCI_HDAT1)){
					//continue_flag = 0;
				}
				*/
            }
            if (TimerCounter16_s == RecordLimit_s){
				continue_flag = 0;
            }

			switch (PinEvent){
			case SWPOFF_PRESSED:
				PCF8523_clearalarm();
				continue_flag = 0;
				PinEvent = 0;
				//lcd_locate(0, 0);
				//lcd_puts((unsigned char *)"SWPOFF_PRESSED  ");
				ret = POWEROFF;
				break;
			case SWSTART_PRESSED:	// 録音停止
				PCF8523_clearalarm();
				continue_flag = 0;
				PinEvent = 0;
				//lcd_locate(0, 0);
				//lcd_puts((unsigned char *)"Record Stop     ");
				ret = STOP_RECORD;
				break;
			//case SWPREV_PRESSED:
			//case SWNEXT_PRESSED:
			default:
				break;
			}
        }

        while(!VS1063_DREQ()) ;
        fillwords = VS1063_ReadSci(SCI_HDAT1);
        for (i=0; i<fillwords; i++){
        	//while(!VS1063_DREQ()) ;        // 不要
            val = VS1063_ReadSci(SCI_HDAT0);
            Mp3Buff[readbytes++] = (val >> 8);
            Mp3Buff[readbytes++] = (val & 0x00FF);
            if (readbytes == ENCDEC_BUF_SIZE){
                res = f_write(file, Mp3Buff, ENCDEC_BUF_SIZE, &bw);
                readbytes = 0;
            }
        }

        val = VS1063_ReadWram(PAR_END_FILL_BYTE);
        if (val & 0x8000){
            f_write(file, &val, 1, &bw);
        }

        //f_truncate(file);
        //filesize = f_size(file);        // bytes
        f_close(file);

        AddMp3fileToSndLst(Mp3FileName, file);
    } else {
    	lcd_lprintln(1, "f_open error    ");
    	//lcd_send_cmd(0xC0);	// move to 2nd line
    	//lcd_puts((unsigned char *)"f_open err");
        //WriteLog("f_open encode outfile error", 0);
    	ret = DO_NOTHING;
    }

	//WriteLog("VS1063_Encode end ", 0);
	return ret;
}

// ファイルのタイムスタンプ
void SetTimestamp(char *Mp3FileName, uint8_t year, uint8_t month, uint8_t day, uint8_t hour, uint8_t minutes, uint8_t sec){
	FILINFO	fno;

	fno.fdate = (WORD)(((year + 2000 - 1980) << 9) | (month << 5) | day);
	fno.ftime = (WORD)((hour << 11) | (minutes << 5) | (sec >> 1));

	f_utime(Mp3FileName, &fno);
}

// MP3ファイルリストファイルにファイル名を追加
void AddMp3fileToSndLst(char *Mp3FileName, FIL *file){
    FRESULT     res;
    uint8_t		buff[RECLEN_MP3LIST];
    UINT        bytes;
    uint16_t	numsnd;
    //int         ret;

    //ret = FR_OK;
    res = f_open(file, SNDFILELISTFILE, FA_READ | FA_WRITE | FA_OPEN_ALWAYS);
	if (res == FR_OK){
		// 先頭レコードを取得
        res = f_read(file, buff, RECLEN_MP3LIST, &bytes);
		if (bytes == 0){
			memset(buff, 0, RECLEN_MP3LIST);
			numsnd = 1;
		} else {
			memcpy(&numsnd, buff, 2);
			numsnd++;
		}
		// MP3ファイル数を先頭レコードに記録する
		memcpy(buff, &numsnd, 2);
		f_lseek(file, 0);
		f_write(file, buff, RECLEN_MP3LIST, &bytes);
		// 末尾にファイル名を追加
		f_lseek(file, RECLEN_MP3LIST * numsnd);
		f_write(file, Mp3FileName, RECLEN_MP3LIST, &bytes);

		f_close(file);
	}
}

// 再生
uint8_t Replay(FIL *file){
    FRESULT     res;
    uint8_t		buff[RECLEN_MP3LIST];
    UINT        bytes;
    uint16_t	numsnd;
    uint16_t    trackno;		// 再生中の曲番 (0,1,2,...,numsndfile-1)
    int         continue_flag;
    int         ret;

    //ret = FR_OK;
    continue_flag = 1;
    res = f_open(file, SNDFILELISTFILE, FA_READ | FA_WRITE | FA_OPEN_ALWAYS);
	if (res == FR_OK){
		// 先頭レコードを取得 -> MP3ファイル数
        res = f_read(file, buff, RECLEN_MP3LIST, &bytes);
		if (bytes > 0){
			memcpy(&numsnd, buff, 2);
		}
		if (numsnd == 0) return 0;
		trackno = numsnd;

		do {
            if (f_lseek(file, (uint32_t)(trackno)*(RECLEN_MP3LIST)) != FR_OK) break;
            if (f_read(file, buff, RECLEN_MP3LIST, &bytes) != FR_OK) break;
			lcd_locate(0, 0);
			lcd_puts(buff);

			// 入力待ち
			ret = PauseAndSelectItem();
			switch (ret){
			case SWDOWN_PRESSED:
			case SWNEXT_PRESSED:
				if (trackno < numsnd) trackno++;
				break;
			case SWUP_PRESSED:
			case SWPREV_PRESSED:
				if (trackno > 1) trackno--;
				break;
			case SWSTART_PRESSED:
				PlayOneMp3((char *)buff, 0, trackno);
				break;
			case SWPOFF_PRESSED:
				continue_flag = 0;
				break;
			}
		} while (continue_flag);

		f_close(file);
	}
	return 0;
}

/*
 * MP3再生
 * 	mp3file:MP3ファイルフルパス、
 */
int PlayOneMp3(const char *mp3file, uint16_t dirno, uint8_t trackno){
    FRESULT     res;
    UINT        sendbytes, sentbytes;
    UINT        bytes;
    int         continue_flag, ret;
    FIL 		filMP3;
    uint16_t	mode;

    while(!VS1063_DREQ()) ;
    mode = SM_SDINEW_B;
    VS1063_WriteSci(SCI_MODE, mode);

    while(!VS1063_DREQ()) ;
    //mode = 0x4000;      // x2.5
    mode = (0x2000 | CLOCKF_FREQ);      // enough to decode MP3 (x2.0)
    //mode = (0x8000 | CLOCKF_FREQ);     // = 0x8000 + ((12000000 - 8000000)/4000)	// VS1063
    //mode = 0x1000;      // too slow
    VS1063_WriteSci(SCI_CLOCKF, mode);          // CLOCKF

    //while(!VS1063_DREQ()) ;
    //VS1063_WriteSci(SCI_AIADDR, 0x50);         // Activation(Patch利用時)

    ret = FR_OK;
    res = f_open(&filMP3, mp3file, FA_READ | FA_OPEN_EXISTING);
    if (res != FR_OK){
    	//DebugWriteln("f_open error in PlayOneMp3 \r\n");
        return res;
    }

    continue_flag = 1;

    /**
    if (PonRestart == 1){       // POWER ON RESTART
    	DWORD	lastbyte;
    	memcpy(&lastbyte, PlayDirHist + HS_LASTBYTE, 4);
        PonRestart = 0;
        //if (f_lseek(&filMP3, *(uint32_t *)(PlayDirHist + HS_LASTBYTE)) != FR_OK){      // この記法はハングする
        if (f_lseek(&filMP3, lastbyte) != FR_OK){      // go to restart point
            f_lseek(&filMP3, 0);
        }
    }
    //VS1011_SetVolume(Volume);
	//DebugWritelnV1("volume st ", Volume);
	**/

    //*(uint16_t *)(PlayDirHist + HS_LASTDIR) = dirno;		// "dereferencing type-punned pointer will break strict-aliasing rules"が出る
    /**
    memcpy(PlayDirHist + HS_LASTDIR, &dirno, 2);
    *(uint16_t *)(PlayDirHist + HS_LASTTRACK) = trackno;
    *(uint32_t *)(PlayDirHist + HS_LASTBYTE) = f_tell(&filMP3);
	**/

    while (continue_flag) {
        res = f_read(&filMP3, Mp3Buff, ENCDEC_BUF_SIZE, &bytes);
        if (res != FR_OK || bytes == 0) break;
        sentbytes = 0;
        while (bytes > 0){
            sendbytes = (bytes > 32 ? 32 : bytes);
            while(!VS1063_DREQ()) ;
            VS1063_DCS_L();
        	SSP_Send(SSP_VS, Mp3Buff + sentbytes, sendbytes);
            VS1063_DCS_H();
            bytes -= sendbytes;
            sentbytes += sendbytes;
        }

        if (PinEvent == SWSTART_PRESSED){
        	//LedOneBlink();
        	PinEvent = 0;
            ret = PLAY_OTHER_DIR;
            continue_flag = 0;
        } else if (PinEvent == SWNEXT_PRESSED){
        	//LedOneBlink();
        	PinEvent = 0;
            ret = PLAY_NEXT_MP3;
            continue_flag = 0;
        } else if (PinEvent == SWPREV_PRESSED){
        	//LedOneBlink();
        	PinEvent = 0;
            ret = PLAY_PREV_MP3;
            continue_flag = 0;
        } else if (PinEvent == SWPOFF_PRESSED){      // POFF
        	//LedOneBlink();
        	PinEvent = 0;
            *(uint32_t *)(PlayDirHist + HS_LASTBYTE) = f_tell(&filMP3);
            ret = POWEROFF;
            break;
        } else if (PinEvent == SWMODE_PRESSED){
        	//LedOneBlink();
        	PinEvent = 0;
            *(uint32_t *)(PlayDirHist + HS_LASTBYTE) = f_tell(&filMP3);
            /**
            if (PlayMode == PM_RANDOM_DIR){
            	ret = GOTO_MAINMENU;
            } else {
            	ret = SELECT_OTHER_MP3;
            }
            **/
            continue_flag = 0;
        }

    }

    f_close(&filMP3);

    return ret;
}

// RTCの調整
int8_t AdjustRtc(){
    char        line1[17];
    char        line2[17];
    char        buff[17];
    uint8_t     val;
    int		    ret;
    int         continue_flag;
    int			i;	//, col;
    uint8_t     pos;    // 日付時刻の桁位置( 0=year,1=month,2=day,3=hour,4=min,5=sec)
    //const uint8_t       startcol = 0;
    const uint8_t		min_col = 0;
    const uint8_t		max_col = 12;
    //const uint8_t		int_col = 6;		// 年月日と時分秒の間に1カラム置く
    static uint8_t      loc[] = {0,2,4,7,9,11};
    static uint8_t      len[] = {2,2,2,2,2,2};
    static uint8_t      min_v[] = {17,1,1,0,0,0};
    static uint8_t      max_v[] = {25,12,31,23,59,59};
    static char         *min_s[] = {"17", "01", "01", "00", "00", "00"};
    static char         *max_s[] = {"25", "12", "31", "23", "59", "59"};
    //LCD_LPrintln(0, "Adjust RTC      ");
    //wait_ms(2000);

    RTC_GetCurrentDateTimeForAdjust(line1);
    ret = 0;
    continue_flag = 1;
    pos = 0;        // year
    //col = min_col;
    memset(line2, ' ', max_col + 1);
	for (i=loc[pos]; i<(loc[pos] + len[pos]); i++){
		line2[i] = '^';
	}
    line2[max_col + 1] = '\0';

    while (continue_flag){
    	// 1行目
    	lcd_locate(0, min_col);
    	lcd_puts((unsigned char *)line1);
    	// 2行目 (カーソル：現在編集対象の日時桁）
    	lcd_locate(1, min_col);
    	lcd_puts((unsigned char *)line2);

        // convert to int
        strncpy(buff, &line1[loc[pos]], len[pos]);
        buff[len[pos]] = '\0';
        val = atoi(buff);

    	//lcd_locate(0, max_col + 1);
    	//lcd_puts("B");

        while (1){	// 無駄なLCD再表示を防ぐためのループ
            ret = PauseAndSelectItem();
            if (ret == SWSELECT_PRESSED){
            	//lcd_locate(1, max_col + 1);
            	//lcd_puts("A");

                RTC_SetDateTime(line1);
                continue_flag = 0;
                ret = SWMODE_PRESSED;
                break;
            } else if (ret == SWNEXT_PRESSED){
            	for (i=loc[pos]; i<(loc[pos] + len[pos]); i++){
            		line2[i] = ' ';
            	}
            	if (pos < 5){
            		pos++;
            	} else {
            		pos = 0;
            	}
            	for (i=loc[pos]; i<(loc[pos] + len[pos]); i++){
            		line2[i] = '^';
            	}
                break;
            } else if (ret == SWPREV_PRESSED){
            	for (i=loc[pos]; i< (loc[pos] + len[pos]); i++){
            		line2[i] = ' ';
            	}
            	if (pos > 0){
            		pos--;
            	} else {
            		pos = 5;	// 秒
            	}
            	for (i=loc[pos]; i< (loc[pos] + len[pos]); i++){
            		line2[i] = '^';
            	}
                break;
            } else if (ret == SWUP_PRESSED){
                if (val < max_v[pos]){
                    val++;
					if (line1[loc[pos] + 1] == '9'){	// １の位が9
						line1[loc[pos] + 1] = '0';
						line1[loc[pos]]++;
					} else {
						line1[loc[pos] + 1]++;
					}
                } else {
                    val = min_v[pos];
                    strncpy(&line1[loc[pos]], min_s[pos], len[pos]);
                }
                break;
            } else if (ret == SWDOWN_PRESSED){
                if (val > min_v[pos]){
                    val--;
					if (line1[loc[pos] + 1] == '0'){
						line1[loc[pos] + 1] = '9';
						line1[loc[pos]]--;
					} else {
						line1[loc[pos] + 1]--;
					}
                } else {
                    val = max_v[pos];
                    strncpy(&line1[loc[pos]], max_s[pos], len[pos]);
                }
                break;
            } else if (ret == SWPOFF_PRESSED){
                continue_flag = 0;
                break;
            //} else {
            }
        }
    }

    lcd_clear();
    return ret;
}

// 現在日時を文字列に取得
uint8_t RTC_GetCurrentDateTimeForAdjust(char *buff){
	uint8_t		year, month, day, hour, minutes, sec;

    PCF8523_gettime(&year, &month, &day, &hour, &minutes, &sec);

    buff[0] = (year / 10) + '0';
    buff[1] = (year % 10) + '0';
    buff[2] = (month / 10) + '0';
    buff[3] = (month % 10) + '0';
    buff[4] = (day / 10) + '0';
    buff[5] = (day % 10) + '0';
    buff[6] = ' ';
    buff[7] = (hour / 10) + '0';
    buff[8] = (hour % 10) + '0';
    buff[9] = (minutes / 10) + '0';
    buff[10] = (minutes % 10) + '0';
    buff[11] = (sec / 10) + '0';
    buff[12] = (sec % 10) + '0';
    buff[13] = '\0';
    return 0;
}

// RTC_GetCurrentDateTimeForAdjustの逆処理。引数datetimeのフォーマットはそちらを参照。
uint8_t RTC_SetDateTime(char *datetime){
	uint8_t		year, month, day, hour, minutes, sec;
	char		buff[4];

	buff[0] = datetime[0];
	buff[1] = datetime[1];
	buff[2] = '\0';
	year = atoi(buff);

	buff[0] = datetime[2];
	buff[1] = datetime[3];
	buff[2] = '\0';
	month = atoi(buff);

	buff[0] = datetime[4];
	buff[1] = datetime[5];
	buff[2] = '\0';
	day = atoi(buff);

	buff[0] = datetime[7];
	buff[1] = datetime[8];
	buff[2] = '\0';
	hour = atoi(buff);

	buff[0] = datetime[9];
	buff[1] = datetime[10];
	buff[2] = '\0';
	minutes = atoi(buff);

	buff[0] = datetime[11];
	buff[1] = datetime[12];
	buff[2] = '\0';
	sec = atoi(buff);

	PCF8523_settime(year, month, day, hour, minutes, sec);

    return 0;
}

/*
uint8_t EncodeSetting(){
	int		ret;
    int     continue_flag = 1;

	do {
		lcd_locate(0, 0);
		lcd_puts(EncSettingStr[CurEncSetting]);

		ret = PauseAndSelectItem();
		switch (ret){
		case SWDOWN_PRESSED:
			if (CurEncSetting > 0) CurEncSetting--;
			else CurEncSetting = NUM_ENCSETTING - 1;
			break;
		case SWUP_PRESSED:
			if (CurEncSetting < (NUM_ENCSETTING - 1)){
				CurEncSetting++;
			} else {
				CurEncSetting = 0;
			}
			break;
		case SWNEXT_PRESSED:
			return SWNEXT_PRESSED;
			break;
		case SWPREV_PRESSED:
			return SWPREV_PRESSED;
			break;
		case SWSELECT_PRESSED:
			//SaveEncodeSettingToEEPROM();
			return SWNEXT_PRESSED;
			break;
		}
	} while (continue_flag);

	return SWNEXT_PRESSED;
}
*/

void LoadPluginFromFile(char *PluginFile, FIL *file) {
    int16_t     i = 0;
    FRESULT     res;
    uint16_t    num;
    UINT        br;

    // get number of data
    res = f_open(file, PluginFile, FA_READ | FA_OPEN_EXISTING);
    if (res != FR_OK){
        //write_eeprom(EP_ADRS_LP_0, res);
    }
    res = f_read(file, &num, 2, &br);
    if (res != FR_OK || br != 2){
        //write_eeprom(EP_ADRS_LP_1, res);
    }

    while (i<num) {
        //  while (i<sizeof(plugin)/sizeof(plugin[0])) {
        uint16_t addr, n, val;
        //    unsigned short addr, n, val;
        //addr = plugin[i++];
        //n = plugin[i++];
        res = f_read(file, &addr, 2, &br);
        if (res != FR_OK){
            //write_eeprom(EP_ADRS_LP_2, res);
        }
        i++;
        res = f_read(file, &n, 2, &br);
        if (res != FR_OK){
            //write_eeprom(EP_ADRS_LP_3, res);
        }
        i++;
        if (n & 0x8000U) { /* RLE run, replicate n samples */
            n &= 0x7FFF;
            //val = plugin[i++];
            res = f_read(file, &val, 2, &br);
            if (res != FR_OK){
                //write_eeprom(EP_ADRS_LP_4, res);
            }
            i++;
            while (n--) {
                while(!VS1063_DREQ()) ;
                //while(!input(MP3_DREQ));
                VS1063_WriteSci(addr, val);
            }
        } else {           /* Copy run, copy n samples */
            while (n--) {
                //val = plugin[i++];
                res = f_read(file, &val, 2, &br);
                if (res != FR_OK){
                    //write_eeprom(EP_ADRS_LP_5, res);
                }
                i++;
                while(!VS1063_DREQ()) ;
                //while(!input(MP3_DREQ));
                VS1063_WriteSci(addr, val);
            }
        }
    }

    f_close(file);

    lcd_lprintln(1, "Load patch      ");
}

// 処理を短時間停止
void WaitMs(uint32_t delayInMs){
	TimerCounter16_0 = 0;
    //LPC_CT16B0->MR0 = 48000 * delayInMs;			// match register = 48000
    //LPC_CT16B0->TCR = 0x01;				//
    //while (LPC_CT16B0->TCR & 0x01);
	while (TimerCounter16_0 < delayInMs){
	}
    //LPC_CT16B0->TCR = 0x02;				//
}

// 割り込みハンドラ(FLEX_INT0_IRQHandlerなどをこれで置き換える) (cr_startup_lpc11uxx.c参照)
void INTx_IRQHandler(void){
	int	bit;

	LedStatus = 1 - LedStatus;

	for (bit=0; bit<=4; bit++){
		if ( LPC_GPIO_PIN_INT->IST & (0x1 << bit) ){		/* Pin Interrupt Status register */
			/* FALL : Pin Interrupt Falling Edge register (detects falling edge) */
			/* IENF : Pin Interrupt Enable Falling Edge / Active Level register */
			if ( LPC_GPIO_PIN_INT->FALL & (0x1 << bit) ){
				LPC_GPIO_PIN_INT->FALL = 0x1 << bit;	/* clears(!) falling edge detect */
			    if (CHATTERING_TMR == 0){
					//PinInterrupt[bit] = 1;
					PinEvent = SW_PRESSED + bit;
		        	CHATTERING_TMR = 1;
			    	//PinEvent = 1 + bit;
			    }
			}
			LPC_GPIO_PIN_INT->IST = 0x1 << bit;
			//DebugWritelnV1("IN PinEvent", PinEvent);
			/*
			{
				char	buff[32], buff2[16];
				strcpy(buff, "INTx_IRQHandler ");
				itoa(bit, buff2, 10);
				strcat(buff, buff2);
				strcat(buff, "\r\n");
				DebugWriteln(buff);
			}
			*/
		}
	}
}

// 割り込みハンドラ（タイマー） (cr_startup_lpc11uxx.c参照)
void TIMER16_0_NEW_IRQHandler(void)
{
	//static uint8_t	event = 0;

	if ( LPC_CT16B0->IR & (0x1<<0) )
	{
		LPC_CT16B0->IR = 0x1<<0;			/* clear interrupt flag */
		TimerCounter16_0++;
		TimerCounter16_ms++;
		if (TimerCounter16_ms == 1000){
			TimerCounter16_ms = 0;
			TimerCounter16_s++;
		}
	}
    if (CHATTERING_TMR == 0){
    	if (GPIOGetPinValue(PORT_SENSE_POWER_DOWN, BIT_SENSE_POWER_DOWN) == 0){
    		//GPIOSetBitValue(PORT_ON_BOARD_LED, BIT_ON_BOARD_LED, 0);
    		PinEvent = SWPOFF_PRESSED;
        	CHATTERING_TMR = 1;
    	}
    } else if (CHATTERING_TMR > 0){
    	CHATTERING_TMR++;
    	if (CHATTERING_TMR == 44){		// 22
    		//GPIOSetBitValue(PORT_ON_BOARD_LED, BIT_ON_BOARD_LED, 1);
    		CHATTERING_TMR = 0;
    	}
    }

#if 0
  if ( LPC_CT16B0->IR & (0x1<<1) )
  {
	LPC_CT16B0->IR = 0x1<<1;			/* clear interrupt flag */
	DebugWriteln("IR 1 ");
	//timer16_0_counter[1]++;
  }
  if ( LPC_CT16B0->IR & (0x1<<4) ) /* Bit4 CR0INT */
  {
	LPC_CT16B0->IR = 0x1<<4;		/* clear interrupt flag */
	//timer16_0_capture[0]++;
  }
  if ( LPC_CT16B0->IR & (0x1<<6) ) /* Bit6 CR1INT */
  {
	LPC_CT16B0->IR = 0x1<<6;		/* clear interrupt flag */
	//timer16_0_capture[1]++;
  }
#endif
  return;
}

// RTC テスト用の初期化
void InitRtc(){
	uint8_t		year, month, day, hour, minutes, sec;
    char        date1[17];

    PCF8523_gettime(&year, &month, &day, &hour, &minutes, &sec);
    if (year != 17){
    	year = 17;
    	month = 11;
    	day = 26;
    	hour = 8;
    	minutes = 30;
    	sec = 0;
		PCF8523_settime(year, month, day, hour, minutes, sec);
		//PCF8523_setalarm(26, 8, 33);
    }
    RTC_GetCurrentDateTimeForAdjust(date1);
    //lcd_locate(1, 0);
    //lcd_puts(date1);
    //Delay(3000);

#if 0
    // set alarm
    minutes += 2;
    if (minutes >= 60){
    	minutes -= 60;
    	hour++;
    	if (hour == 24){
    		hour = 0;
    		day++;
    	}
    }
	PCF8523_setalarm(day, hour, minutes);
#endif
}

// 録音スケジュールをRTCにセット
// スケジュールファイルはMP3_05と共通(レコード例：
// 170414:060000:170414:065500:F:8250:0:5)
// 01234567890123456789012345678901234567
uint8_t SetNextSchedule(FIL *file){
    FRESULT     res;
    char        buff1[64];
    char		buff2[32];
	uint8_t		eeprom[EEPROM_LEN_SCHEDULE];
	uint8_t		year, month, day, hour, minutes, sec;
	uint8_t		year_sch, month_sch, day_sch, hour_sch, minutes_sch;
	uint8_t		next_sch;

    res = f_open(file, SCHEDULEFILE, FA_READ | FA_OPEN_EXISTING);
    if (res == FR_OK){
    	lcd_lprintln(0, "Load Schedule   ");
        PCF8523_gettime(&year, &month, &day, &hour, &minutes, &sec);
        next_sch = 0;

        while (f_gets(buff1, 64, file) != NULL){
            if (buff1[0] == '#') continue;

            if (buff1[6] == ':' && buff1[13] == ':' && buff1[20] == ':' && buff1[27] == ':'){
            	// スケジュール：年
            	buff2[0] = buff1[0];
            	buff2[1] = buff1[1];
            	buff2[2] = '\0';
            	year_sch = atoi(buff2);
            	// スケジュール：月
            	buff2[0] = buff1[2];
            	buff2[1] = buff1[3];
            	buff2[2] = '\0';
            	month_sch = atoi(buff2);
            	// スケジュール：日
            	buff2[0] = buff1[4];
            	buff2[1] = buff1[5];
            	buff2[2] = '\0';
            	day_sch = atoi(buff2);
            	// スケジュール：時
            	buff2[0] = buff1[7];
            	buff2[1] = buff1[8];
            	buff2[2] = '\0';
            	hour_sch = atoi(buff2);
            	// スケジュール：分
            	buff2[0] = buff1[9];
            	buff2[1] = buff1[10];
            	buff2[2] = '\0';
            	minutes_sch = atoi(buff2);

            	if ((year_sch == year && month_sch == month && day_sch == day && hour_sch == hour && minutes_sch >= minutes) ||
            		(year_sch == year && month_sch == month && day_sch == day && hour_sch > hour) ||
            		(year_sch == year && month_sch == month && day_sch > day) ||
            		(year_sch == year && month_sch > month) ||
            		(year_sch > year)){	// 現在時刻より後の

            		next_sch = 1;
            		PCF8523_setalarm(day_sch, hour_sch, minutes_sch);
            		// EEPROM保存
            		eeprom[0] = buff1[0];	// 開始・年
            		eeprom[1] = buff1[1];
            		eeprom[2] = '\0';
            		eeprom[3] = buff1[2];	// 開始・月
            		eeprom[4] = buff1[3];
            		eeprom[5] = '\0';
            		eeprom[6] = buff1[4];	// 開始・日
            		eeprom[7] = buff1[5];
            		eeprom[8] = '\0';
            		eeprom[9] = buff1[7];	// 開始・時
            		eeprom[10] = buff1[8];
            		eeprom[11] = '\0';
            		eeprom[12] = buff1[9];	// 開始・分
            		eeprom[13] = buff1[10];
            		eeprom[14] = '\0';

            		eeprom[15] = buff1[14];	// 終了・年
            		eeprom[16] = buff1[15];
            		eeprom[17] = '\0';
            		eeprom[18] = buff1[16];	// 終了・月
            		eeprom[19] = buff1[17];
            		eeprom[20] = '\0';
            		eeprom[21] = buff1[18];	// 終了・日
            		eeprom[22] = buff1[19];
            		eeprom[23] = '\0';
            		eeprom[24] = buff1[21];	// 終了・時
            		eeprom[25] = buff1[22];
            		eeprom[26] = '\0';
            		eeprom[27] = buff1[23];	// 終了・分
            		eeprom[28] = buff1[24];
            		eeprom[29] = '\0';
            		iap_eeprom_write(EEPROM_ADDR_SCHEDULE, eeprom, EEPROM_LEN_SCHEDULE);

            		// 表示
            		lcd_locate(0, 0);
            		lcd_puts((unsigned char *)"Next Schedule:");
            		lcd_locate(1, 0);
            		buff2[0] = buff1[4];
            		buff2[1] = buff1[5];
            		buff2[2] = buff1[7];
            		buff2[3] = buff1[8];
            		buff2[4] = buff1[9];
            		buff2[5] = buff1[10];
            		buff2[6] = '-';
            		buff2[7] = buff1[18];
            		buff2[8] = buff1[19];
            		buff2[9] = buff1[21];
            		buff2[10] = buff1[22];
            		buff2[11] = buff1[23];
            		buff2[12] = buff1[24];
            		buff2[13] = '\0';
            		lcd_puts((unsigned char *)buff2);
    				WaitMs(5000);

            		// EEPROM確認
            	    memset(eeprom, 0, EEPROM_LEN_SCHEDULE);
            		iap_eeprom_read(EEPROM_ADDR_SCHEDULE, eeprom, EEPROM_LEN_SCHEDULE);
            		lcd_locate(0, 0);
            		lcd_puts((unsigned char *)"End Schedule:");
            		lcd_locate(1, 0);
            		buff2[0] = eeprom[15];
            		buff2[1] = eeprom[16];
            		buff2[2] = eeprom[18];
            		buff2[3] = eeprom[19];
            		buff2[4] = eeprom[21];
            		buff2[5] = eeprom[22];
            		buff2[6] = eeprom[24];
            		buff2[7] = eeprom[25];
            		buff2[8] = eeprom[27];
            		buff2[9] = eeprom[28];
            		buff2[10] = ' ';
            		lcd_puts((unsigned char *)buff2);
    				WaitMs(5000);

            		break;
            	}
            } else {
        		lcd_lprintln(1, "Schedule Error  ");
        		WaitMs(3000);
            }
        }
        f_close(file);
        if (next_sch == 0){
			PCF8523_clearalarm();
    		lcd_lprintln(1, "No Schedule     ");
    		WaitMs(3000);
        }
    } else {
		PCF8523_clearalarm();
		lcd_lprintln(1, "No Schedule File");
		WaitMs(3000);
    }

    return 0;
}

int iap_eeprom_write(void* eeprom_address, void* ram_address, unsigned int count) {
    unsigned int command[5];
    unsigned int result[4];

    command[0] = 61;
    command[1] = (unsigned int) eeprom_address;
    command[2] = (unsigned int) ram_address;
    command[3] = count;
    command[4] = SystemCoreClock / 1000;
    iap_entry(command, result);

    return (int) result[0];
}

int iap_eeprom_read(void* eeprom_address, void* ram_address, unsigned int count) {
    unsigned int command[5];
    unsigned int result[4];

    command[0] = 62;
    command[1] = (unsigned int) eeprom_address;
    command[2] = (unsigned int) ram_address;
    command[3] = count;
    command[4] = SystemCoreClock / 1000;
    iap_entry(command, result);

    return (int) result[0];
}

