/*
 * rtc.c
 *
 *  Created on: 2017/11/26
 *      Author: yamada
 */

#include <stdint.h>
#include "type.h"
#include "gpio.h"
//#include "uart.h"
//#include "ssp.h"
#include "i2c.h"
#include "stdlib.h"
#include "mp3_11.h"

#define PCF8523_ADDRESS       (0x68 << 1)
#define PCF8523_CLKOUTCONTROL 0x0F
#define PCF8523_CONTROL_1     0x00
#define PCF8523_CONTROL_2     0x01
#define PCF8523_CONTROL_3     0x02
#define PCF8523_REG_SECOND    0x03
#define PCF8523_ALARM_SECOND  0x0A
#define PCF8523_TMR_CLKOUT_CTRL 0x0F

uint8_t bcd2bin (uint8_t val) { return val - 6 * (val >> 4); }
uint8_t bin2bcd (uint8_t val) { return val + 6 * (val / 10); }

void PCF8523_settime(uint8_t year, uint8_t month, uint8_t day, uint8_t hour, uint8_t min, uint8_t sec){
	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = PCF8523_ADDRESS;
	I2CMasterBuffer[1] = PCF8523_TMR_CLKOUT_CTRL;		/* address */
	I2CMasterBuffer[2] = 0x38;		/* Data0 */
	I2CEngine();

#if 0
    i2c_start();
    i2c_write(PCF8523_ADDRESS);
    i2c_write(PCF8523_TMR_CLKOUT_CTRL);
    i2c_write(0x38);    // CLKOUT disabled
    i2c_stop();
#endif

	I2CWriteLength = 9;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = PCF8523_ADDRESS;
	I2CMasterBuffer[1] = PCF8523_REG_SECOND;
	I2CMasterBuffer[2] = bin2bcd(sec);
	I2CMasterBuffer[3] = bin2bcd(min);
	I2CMasterBuffer[4] = bin2bcd(hour);
	I2CMasterBuffer[5] = bin2bcd(day);
	I2CMasterBuffer[6] = 0;		// 曜日
	I2CMasterBuffer[7] = bin2bcd(month);
	I2CMasterBuffer[8] = bin2bcd(year);
	I2CEngine();

#if 0
    i2c_start();
    i2c_write(PCF8523_ADDRESS);
    i2c_write(PCF8523_REG_SECOND);
    i2c_write(bin2bcd(sec));
    i2c_write(bin2bcd(min));
    i2c_write(bin2bcd(hour));
    i2c_write(bin2bcd(day));
    i2c_write(0);
    i2c_write(bin2bcd(month));
    i2c_write(bin2bcd(year));
    i2c_stop();
#endif

}

void PCF8523_setalarm(uint8_t day, uint8_t hour, uint8_t min){
	I2CWriteLength = 6;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = PCF8523_ADDRESS;
	I2CMasterBuffer[1] = PCF8523_ALARM_SECOND;
	I2CMasterBuffer[2] = bin2bcd(min);
	I2CMasterBuffer[3] = bin2bcd(hour);
	I2CMasterBuffer[4] = bin2bcd(day);
	I2CMasterBuffer[5] = 0x80;
	I2CEngine();

#if 0
    i2c_start();
    i2c_write(PCF8523_ADDRESS);
    i2c_write(PCF8523_ALARM_SECOND);
    i2c_write(bin2bcd(min));
    i2c_write(bin2bcd(hour));
    i2c_write(bin2bcd(day));
    i2c_stop();
#endif

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = PCF8523_ADDRESS;
	I2CMasterBuffer[1] = PCF8523_CONTROL_1;
	I2CMasterBuffer[2] = 0x02;		// set AIE (alarm interrupt enabled)
	I2CEngine();

#if 0
    i2c_start();
    i2c_write(PCF8523_ADDRESS);
    i2c_write(PCF8523_CONTROL_1);
    i2c_write(0x02);    // set AIE
    i2c_stop();
#endif

	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = PCF8523_ADDRESS;
	I2CMasterBuffer[1] = PCF8523_CONTROL_3;
	I2CMasterBuffer[2] = 0x80;		// PM[2:0] = 100 (battery switch-over function is enabled in standard mode)
	I2CEngine();
}

void PCF8523_clearalarm(){
	// Get CONTROL_2
	I2CWriteLength = 2;
	I2CReadLength = 1;
	I2CMasterBuffer[0] = PCF8523_ADDRESS;
	I2CMasterBuffer[1] = PCF8523_CONTROL_2;
	I2CMasterBuffer[2] = (PCF8523_ADDRESS | 0x01);
	I2CEngine();

	// Clear AF
	I2CWriteLength = 3;
	I2CReadLength = 0;
	I2CMasterBuffer[0] = PCF8523_ADDRESS;
	I2CMasterBuffer[1] = PCF8523_CONTROL_2;
	I2CMasterBuffer[2] = (I2CSlaveBuffer[0] & ~0x08);		// clear AF
	I2CEngine();

#if 0
    i2c_start();
    i2c_write(PCF8523_ADDRESS);
    i2c_write(PCF8523_CONTROL_2);
    i2c_write(0x00);    // clear AF
    i2c_stop();
#endif
}

uint8_t PCF8523_isalarmed(){
	I2CWriteLength = 2;
	I2CReadLength = 1;
	I2CMasterBuffer[0] = PCF8523_ADDRESS;
	I2CMasterBuffer[1] = PCF8523_CONTROL_2;
	I2CMasterBuffer[2] = (PCF8523_ADDRESS | 0x01);
	I2CEngine();

	if ((I2CSlaveBuffer[0] & 0x08) != 0){
		return 1;
	} else {
		return 0;
	}
}


void PCF8523_gettime(uint8_t *year, uint8_t *month, uint8_t *day, uint8_t *hour, uint8_t *min, uint8_t *sec){
	I2CWriteLength = 2;
	I2CReadLength = 7;
	I2CMasterBuffer[0] = PCF8523_ADDRESS;
	I2CMasterBuffer[1] = PCF8523_REG_SECOND;		/* address */
	//I2CMasterBuffer[2] = 0x38;		/* Data0 */
	//I2CMasterBuffer[3] = (PCF8523_ADDRESS | 0x01);
	I2CMasterBuffer[2] = (PCF8523_ADDRESS | 0x01);
	//I2CEngine();

#if 0
	i2c_start();
    i2c_write(PCF8523_ADDRESS);
    i2c_write(PCF8523_REG_SECOND);
    i2c_stop();
#endif

	//I2CWriteLength = 1;
	//I2CReadLength = 7;
	//I2CMasterBuffer[0] = PCF8523_ADDRESS | 0x01;
	I2CEngine();

    *sec = bcd2bin(I2CSlaveBuffer[0]);
    *min = bcd2bin(I2CSlaveBuffer[1]);
    *hour = bcd2bin(I2CSlaveBuffer[2]);
    *day = bcd2bin(I2CSlaveBuffer[3]);
    //I2CSlaveBuffer[4];	// 曜日
    *month = bcd2bin(I2CSlaveBuffer[5]);
    *year = bcd2bin(I2CSlaveBuffer[6]);

#if 0
    i2c_start();
    i2c_write(PCF8523_ADDRESS | 0x01);
    *sec = bcd2bin(i2c_read(TRUE));
    *min = bcd2bin(i2c_read(TRUE));
    *hour = bcd2bin(i2c_read(TRUE));
    *day = bcd2bin(i2c_read(TRUE));
    i2c_read(TRUE);
    *month = bcd2bin(i2c_read(TRUE));
    *year = bcd2bin(i2c_read(FALSE));
    i2c_stop();
#endif
}

#if 0
int PCF8523_initialized() {
    i2c_start();
    i2c_write(PCF8523_ADDRESS);
    i2c_write(PCF8523_CONTROL_3);
    i2c_stop();

    return 0;
}
#endif


